@@ -8,9 +8,9 @@ require Msql::Statement;
 $QUIET  = $QUIET  = '';
 @ISA    = @ISA    = '';
 @EXPORT = @EXPORT = '';
-$VERSION = $VERSION = "1.1703";
+$VERSION = $VERSION = "1.1801";
 
-# $Revision: 1.105 $$Date: 1997/06/05 17:00:15 $$RCSfile: Msql.pm,v $
+# $Revision: 1.1.1.1 $$Date: 1997/08/27 10:31:58 $$RCSfile: Msql.pm,v $
 
 $QUIET = 0;
 
@@ -37,20 +37,9 @@ use Exporter ();
 		SYSVAR_TYPE
 	       );
 
-sub INT_TYPE    { constant("INT_TYPE", 0) }
-sub CHAR_TYPE   { constant("CHAR_TYPE", 0) }
-sub REAL_TYPE   { constant("REAL_TYPE", 0) }
-sub IDENT_TYPE  { constant("IDENT_TYPE", 0) }
-sub NULL_TYPE   { constant("NULL_TYPE", 0) }
-sub TEXT_TYPE   { constant("TEXT_TYPE", 0) }
-sub DATE_TYPE   { constant("DATE_TYPE", 0) }
-sub UINT_TYPE   { constant("UINT_TYPE", 0) }
-sub MONEY_TYPE  { constant("MONEY_TYPE", 0) }
-sub TIME_TYPE   { constant("TIME_TYPE", 0) }
-sub IDX_TYPE    { constant("IDX_TYPE", 0) }
-sub SYSVAR_TYPE { constant("SYSVAR_TYPE", 0) }
 sub host     { return shift->{'HOST'} }
 sub sock     { return shift->{'SOCK'} }
+sub sockfd   { return shift->{'SOCKFD'} }
 sub database { return shift->{'DATABASE'} }
 
 sub quote	{
@@ -65,23 +54,43 @@ sub quote	{
 
 sub AUTOLOAD {
     my $meth = $AUTOLOAD;
-    $meth =~ s/^.*:://;
-    $meth =~ s/_//g;
-    $meth = lc($meth);
+    my $converted = 0;
+
+    if ($meth =~ /(.*)::/) {
+	$meth = $';
+	$class = $1;
+    } else {
+	$class = "main";
+    }
+
 
     TRY: {
-	  if (defined &$meth) {
-	      *$meth = \&{$meth};
-	      return &$meth(@_);
-	  } elsif ($meth =~ s/(.*)type$/uc($1)."_TYPE"/e) {
-	      # Try to determine the type that was requested by
-	      # translating inttype to INT_TYPE Not that I consider it
-	      # good style to write inttype, but we once allowed it,
-	      # so...
-	      redo TRY;
-	  }
-      }
-    Carp::croak "$AUTOLOAD: Not defined in Msql and not autoloadable (last try $meth)";
+	my $val = constant($meth, @_ ? $_[0] : 0);
+	if ($! == 0) {
+	    eval "sub $AUTOLOAD { $val }";
+	    return $val;
+	}
+
+	if (!$converted) {
+	    $meth =~ s/_//g;
+	    $meth = lc($meth);
+	    $converted = 1;
+	}
+
+	if (defined &$meth) {
+	    *$meth = \&{$meth};
+	    return &$meth(@_);
+	} elsif ($meth =~ s/(.*)type$/uc($1)."_TYPE"/e) {
+	    # Try to determine the type that was requested by
+	    # translating inttype to INT_TYPE Not that I consider it
+	    # good style to write inttype, but we once allowed it,
+	    # so...
+	    redo TRY;
+	}
+    }
+
+  Carp::croak "$AUTOLOAD: Not defined in $class and not"
+      . " autoloadable (last try $meth)";
 }
 
 bootstrap Msql;
@@ -91,7 +100,7 @@ __END__
 
 =head1 NAME
 
-Msql - Perl interface to the mSQL database
+Msql / Mysql - Perl interfaces to the mSQL and mysql databases
 
 =head1 SYNOPSIS
 
@@ -100,6 +109,13 @@ Msql - Perl interface to the mSQL database
   $dbh = Msql->connect;
   $dbh = Msql->connect($host);
   $dbh = Msql->connect($host, $database);
+
+      or
+
+  use Mysql;
+
+  $dbh = Mysql->connect(undef, $database, $user, $password);
+  $dbh = Mysql->connect($host, $database, $user, $password);
 	
   $dbh->selectdb($database);
 	
@@ -113,6 +129,7 @@ Msql - Perl interface to the mSQL database
   $sth = $dbh->query($sql_statement);
 	
   @arr = $sth->fetchrow;
+  @arr = $sth->fetchcol($col_number);
   %hash = $sth->fetchhash;
 	
   $sth->dataseek($row_number);
@@ -126,23 +143,33 @@ Msql - Perl interface to the mSQL database
 =head1 DESCRIPTION
 
 This package is designed as close as possible to its C API
-counterpart. The manual that comes with mSQL describes most things you
-need. Due to popular demand it was decided though, that this interface
+counterpart. The manual that comes with mSQL or MySQL describes most things
+you need. Due to popular demand it was decided though, that this interface
 does not use StudlyCaps (see below).
 
+The version you have selected is an adaption still under development,
+please consult the file "Changes" in your distribution.
+
 Internally you are dealing with the two classes C<Msql> and
-C<Msql::Statement>. You will never see the latter, because you reach
+C<Msql::Statement> or C<Mysql> and C<Mysql::Statement>, respectively.
+You will never see the latter, because you reach
 it through a statement handle returned by a query or a listfields
-statement. The only class you name explicitly is Msql. It offers you
-the connect command:
+statement. The only class you name explicitly is Msql or Mysql. They
+offer you the connect command:
 
   $dbh = Msql->connect;
   $dbh = Msql->connect($host);
   $dbh = Msql->connect($host, $database);
 
+    or
+
+  $dbh = Msql->connect(undef, undef, $user, $password);
+  $dbh = Msql->connect($host, undef, $user, $password);
+  $dbh = Msql->connect($host, $database, $user, $password);
+
 This connects you with the desired host/database. With no argument or
 with an empty string as the first argument it connects to the UNIX
-socket (usually /dev/msql), which has a much better performance than
+socket, which has a much better performance than
 the TCP counterpart. A database name as the second argument selects
 the chosen database within the connection. The return value is a
 database handle if the connect succeeds, otherwise the return value is
@@ -160,7 +187,7 @@ selectdb.
   $sth = $dbh->listfields($table);
   $sth = $dbh->query($sql_statement);
 
-These two work rather similar as descibed in the mSQL manual. They
+These two work rather similar as descibed in the mSQL or MySQL manual. They
 return a statement handle which lets you further explore what the
 server has to tell you. On error the return value is undef. The object
 returned by listfields will not know about the size of the table, so a
@@ -185,6 +212,13 @@ have a table with two identical column names, you will not be able to
 use this method without trashing one column. In such a case, you
 should use the fetchrow method.
 
+  @arr = $sth->fetchcol($colnum);
+
+returns an array of the values of each row for column $colnum.  Note that
+this reads the entire table and leaves the row offset at the end of the
+table; be sure to use $sth->dataseek() to reset it if you want to
+re-examine the table.
+
   $sth->dataseek($row_number);
 
 lets you specify a certain offset of the data associated with the
@@ -220,7 +254,7 @@ executed command, the latter not.
 
 =head2 The C<-w> switch
 
-With Msql the C<-w> switch is your friend! If you call your perl
+With Msql and Mysql the C<-w> switch is your friend! If you call your perl
 program with the C<-w> switch you get the warnings from -E<gt>errmsg on
 STDERR. This is a handy method to get the error messages from the msql
 server without coding it into your program.
@@ -230,17 +264,21 @@ environment variables that are described in David's manual. David's
 debugging aid is excellent, there's nothing to be added.
 
 If you want to use the C<-w> switch but do not want to see the error
-messages from the msql daemon, you can set the variable $Msql::QUIET
-to some true value, and they will be supressed.
+messages from the msql daemon, you can set the variables $Msql::QUIET
+or $Mysql::QUIET to some true value, and they will be supressed.
 
 =head2 -E<gt>quote($str [, $length])
 
 returns the argument enclosed in single ticks ('') with any special
-character escaped according to the needs of the API. Currently this
-means, any single tick within the string is escaped with a backslash
-and backslashes are doubled. Currently (as of msql-1.0.16) the API
-does not allow to insert binary nulls into tables. The quote method
-does not fix this deficiency.
+character escaped according to the needs of the API.
+
+For mSQL this means, any single tick within the string is escaped with
+a backslash and backslashes are doubled. Currently (as of msql-1.0.16)
+the API does not allow to insert NUL's (ASCII 0) into tables. The quote
+method does not fix this deficiency.
+
+MySQL allows NUL's or any other kind of binary data in strings. Thus
+the quote method will additionally escape NUL's as \0.
 
 If you pass undefined values to the quote method, it returns the
 string C<NULL>.
@@ -262,6 +300,10 @@ As said above you get a database handle with
 
   $dbh = Msql->connect($host, $database);
 
+    or
+
+  $dbh = Mysql->connect($host, $database);
+
 The database handle knows about the socket, the host, and the database
 it is connected to.
 
@@ -271,8 +313,13 @@ You get at the three values with the methods
   $scalar = $dbh->host;
   $scalar = $dbh->database;
 
-database returns undef, if you have connected without or with only one
-argument.
+Mysql additionally supports
+
+  $scalar = $dbh->user;
+  $scalar = $dbh->sockdf;
+
+where the latter is the file descriptor of the socket used by the
+database  connection. This is the same as $dbh->sock for mSQL.
 
 =head2 Statement Handle
 
@@ -290,15 +337,26 @@ $sth knows about all metadata that are provided by the API:
   @arr  = $sth->name;        the names of the columns
   @arr  = $sth->type;        the type of each column, defined in msql.h
 	                     and accessible via Msql::CHAR_TYPE,
-	                     &Msql::INT_TYPE, &Msql::REAL_TYPE,
-  @arr  = $sth->isnotnull;   array of boolean
-  @arr  = $sth->isprikey;    array of boolean
+	                     &Msql::INT_TYPE, &Msql::REAL_TYPE or
+                             &Mysql::FIELD_TYPE_STRING,
+                             &Mysql::FIELD_TYPE_LONG, ...
+  @arr  = $sth->is_not_null; array of boolean
+  @arr  = $sth->is_pri_key;  array of boolean
   @arr  = $sth->length;      array of the length of each field in bytes
 
-The six last methods return an array in array context and an array
-reference (see L<perlref> and L<perlldsc> for details) when called in
-a scalar context. The scalar context is useful, if you need only the
-name of one column, e.g.
+Mysql additionally supports
+
+  $scalar  = $sth->affected_rows number of rows in database affected by query
+  $scalar  = $sth->insert_id     the unique id given to a auto_increment field.
+  $string  = $sth->info()        more info from some queries (ALTER TABLE...)
+  $arrref  = $sth->is_num;       array of boolean
+  $arrref  = $sth->is_blob;      array of boolean
+
+The array methods (table, name, type, is_not_null, is_pri_key, length,
+affected_rows, is_num and blob) return an array in array context and
+an array reference (see L<perlref> and L<perlldsc> for details) when
+called in a scalar context. The scalar context is useful, if you need
+only the name of one column, e.g.
 
     $name_of_third_column = $sth->name->[2]
 
@@ -326,10 +384,10 @@ C<listindices()> method on a statement handle. To find out the columns
 included in an index, you can call the C<listindex($table,$index)>
 method on a database handle.
 
-There are a few new column types in mSQL 2. Access their numeric value
-with the these functions defined in the Msql package: IDENT_TYPE,
-NULL_TYPE, TEXT_TYPE, DATE_TYPE, UINT_TYPE, MONEY_TYPE, TIME_TYPE,
-IDX_TYPE, SYSVAR_TYPE.
+There are a few new column types in mSQL 2. You can access their
+numeric value with these functions defined in the Msql package:
+IDENT_TYPE, NULL_TYPE, TEXT_TYPE, DATE_TYPE, UINT_TYPE, MONEY_TYPE,
+TIME_TYPE, IDX_TYPE, SYSVAR_TYPE.
 
 You cannot talk to a 1.0 server with a 2.0 client.
 
@@ -348,21 +406,6 @@ always have them imported into your namespace. I consider it a bug,
 but not such a serious one, that I intend to break old programs by
 moving them into EXPORT_OK.
 
-=head2 Connecting to different port(s)
-
-The mSQL API allows you to interface to a different port than the
-default that is compiled into your copy. To use this feature you have
-to set the environment variable MSQL_TCP_PORT. You can do so at any
-time in your program with the command
-
-    $ENV{'MSQL_TCP_PORT'} = 1234; # or 1112 or 1113 or 4333 or 4334
-
-Any subsequent connect() will establish a connection to the specified
-port.
-
-For connect()s to the UNIX socket of the local machine use
-MSQL_UNIX_PORT instead.
-
 =head2 Displaying whole tables in one go
 
 A handy method to show the complete contents of a statement handle is
@@ -429,8 +472,8 @@ Output of pmsql:
 
 =head2 Version information
 
-The version of MsqlPerl is always stored in $Msql::VERSION as it is
-perl standard.
+The version of Msql and Mysql is always stored in $Msql::VERSION or
+$Mysql::VERSION as it is perl standard.
 
 The mSQL API implements methods to access some internal configuration
 parameters: gethostinfo, getserverinfo, and getprotoinfo.  All three
@@ -440,14 +483,17 @@ three return global variables that reflect the B<last> connect()
 command within the current program. This means, that all three return
 empty strings or zero I<before> the first call to connect().
 
+This situation is better with MySQL: The methods are valid only
+in connection with a datanase handle.
+
 =head2 Administration
 
 shutdown, createdb, dropdb, reloadacls are all accessible via a
 database handle and implement the corresponding methods to what
 msqladmin does.
 
-The mSQL engine does not permit that these commands are invoked by
-other users than the administrator ofthe database. So please make sure
+The mSQL and MySQL engines do not permit that these commands are invoked by
+users without sufficient privileges. So please make sure
 to check the return and error code when you issue one of them.
 
 =head2 StudlyCaps
@@ -471,6 +517,10 @@ work.
 mSQL is a database server and an API library written by David
 Hughes. To use the adaptor you definitely have to install these first.
 
+MySQL is a libmysqlclient.a library written by Michael Widenius
+This was originally inspired by MySQL.
+
+
 =head1 AUTHOR
 
 andreas koenig C<koenig@franz.ww.TU-Berlin.DE>
@@ -486,4 +536,3 @@ with better support and more functionality. Alligator maintains an
 interesting page on the DBI development: http://www.hermetica.com/
 
 =cut
-
